@@ -8,7 +8,7 @@ HAVE_RAND = have_func('rand')
 HAVE_SRAND = have_func('srand')
 
 def sized_int(size, types)
-  types.find { |type| check_sizeof(type) == 4 } ||
+  types.find { |type| check_sizeof(type) == size } ||
     abort("no int with size #{size}")
 end
 
@@ -33,6 +33,7 @@ open(File.join(File.dirname(__FILE__), "ruby-config.h"), "wb") do |f|
 end
 
 $defs.push("-DVERSION=\\\"#{VERSION}\\\"")
+$defs.push("-DBRANCH=\"\"")
 
 # Post XCode 5.1 the command line tools on OS X treat unrecognised 
 # command line options as errors and it's been seen that
@@ -42,4 +43,8 @@ if /darwin|mac os/.match RbConfig::CONFIG['host_os']
   $DLDFLAGS.gsub!("-multiply_definedsuppress", "")
 end
 
+if /mswin/.match RbConfig::CONFIG['host_os']
+  $defs.push("-Dinline=__inline")
+end
+
 create_makefile('rdiscount')
